#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "uart.h"
#include "string.h"

#include "flasher.h"

void setAtmega()    //nastaveni atmega
{
	//vstup/vstup porty
   	DDRB|=(1<<PORTB3);
	DDRD|=(1<<PORTD7) | (1<<PORTD5);
   	
	//zapnut pull-up
	PORTD |= (1 << PORTD3);
	//preruseni   
	MCUCR |= (1 << ISC11) | (0 << ISC10) ;
	GICR |= (1 << INT1);
	
	//pwm
	TCCR1B |= (1<<CS10);
	TIMSK |= (1 << TOIE1) | (1<<OCIE1A) | (1<<OCIE1B);
	OCR1A = 63036;
	OCR1B = 63036;
	
	//peruen OVF
	TCCR2|=(1<<CS22)|(1<<CS21)|(1<<CS20);
	TIMSK |=(1<<TOIE2);
	
	//power down mod
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	
}

int dipSwitchRead()     //fce na cteni nastaveni kodoveho prepinace
{
	DDRA = 0x00;
	PORTA |= (1 << PORTA1);
	PORTA |= (1 << PORTA2);
	if ((PINA & (1 << PD1) ) && (PINA & (1 << PD2) )) return 3;
	else if (PINA & (1 << PD2) ) return 2;
	else if (PINA & (1 << PD1) ) return 1;
	else return 0;
}

void initADC()        //fce na iniciaci ADC
{
	ADMUX=(1<<REFS0);                        
	ADCSRA=(1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0); //peddlika na /128
}

uint16_t readADC(uint8_t ch)    //fce na obsluhu ADC
{
	_delay_ms(10);
	ch=ch&0b00000111;
	ADMUX|=ch;
	ADCSRA|=(1<<ADSC);
	while(!(ADCSRA & (1<<ADIF)));

	ADCSRA|=(1<<ADIF);
	_delay_ms(10);
	return(ADC);
}

void yellowLightOn(int Intensity)      // fce na ovladani zlutych svetel
{
	OCR1A = Intensity*10 + 63036;

}

void redLightOn(int Intensity)   // fce na ovladani rervenych svetel
{
	OCR1B = Intensity + 63036;
}


void sleep()      //funkce na uspani AVR
{
	yellowLightOn(1);
	redLightOn(1);
	mpu6050_setSleepEnabled();
	sleep_mode();
}

void brakeLight(double x, double y, double z, float b)   //fce na kotrolu a roysviceni brydovych svetel
{
	z -=sqrt(1 - x*x - y*y);
	if (z<-b) 
	{
		redLightOn(2500);
	}
	else
	{
		redLightOn(200);
	}
}

int bikePosition(int16_t x, int16_t y, int16_t z, float p)      //kotrola pozice jizdniho kola
{
	if (x>p || x<-p) return 1;
	else return 0; 
}