#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <math.h> 
#include <avr/wdt.h>
#include <avr/sleep.h>

#include "mpu6050/mpu6050.h"
#include "mpu6050/mpu6050.c"

#include "flasher.h"
#include "flasher.c"

 //nastaveni volatilnich promennych pouzivanych v preruseni
volatile unsigned int q = 0;
volatile unsigned int b = 0;

//kalibracni promenne
uint8_t accelCalibration = 0x00;
uint8_t gyroCalibration = 0x00;
float aGain;
float gGain;


int main(void) {
  //nezbytne promenne
	uint16_t adc;
	float pos;
	float posConv;
	int btnConf;

	int16_t ax = 0;
	int16_t ay = 0;
	int16_t az = 0;
	int16_t gx = 0;
	int16_t gy = 0;
	int16_t gz = 0;
	double axg = 0;
	double ayg = 0;
	double azg = 0;
	double gxds = 0;
	double gyds = 0;
	double gzds = 0;


  //povoleni preruseni
	sei();
	//cteni nastaveni kodoveho prepinace
	btnConf = dipSwitchRead();
  //konfigurace mpu
	switch (btnConf)
	{
		case 0:
		gyroCalibration = 0x00;
		accelCalibration = 0x00;
		aGain = 16384.0 ;
		gGain = 131;
		break;
		case 1:
		accelCalibration = 0x01;
		gyroCalibration = 0x01;
		aGain = 8192.0;
		gGain = 65.5;
		break;
		case 2:
		accelCalibration = 0x02;
		gyroCalibration = 0x02;
		aGain = 4096.0;
		gGain = 32.8;
		break;
		case 3:
		accelCalibration = 0x03;
		gyroCalibration = 0x03;
		aGain = 2048.0;
		gGain = 16.4;
		break;
	}
	
	pos = 9000/(btnConf+1);
	mpu6050_init(gyroCalibration, accelCalibration);
	
	_delay_ms(100);
  //resetovani a inicializace watchdogu
	wdt_reset();
	wdt_enable(WDTO_2S);
  //kontrola polohy jiydniho kola
	mpu6050_getRawData(&ax, &ay, &az, &gx, &gy, &gz);
	if (!bikePosition(ax, ay, az, pos)) sleep();
  //inicializace ADC a nastaveni Atmegy
	initADC();
	setAtmega();
	_delay_ms(50);
	
	while(1)
   {
		wdt_reset();
		//ziskani dat z mpu
    mpu6050_getRawData(&ax, &ay, &az, &gx, &gy, &gz);
		mpu6050_getConvData(&axg, &ayg, &azg, &gxds, &gyds, &gzds, aGain, gGain);
		// ziskani dat z ADC o intenzite okolniho osvetleni a rozsviceni zlutych svetel
		adc = readADC(0);
		yellowLightOn(adc-774);
    //kotrola brzdeni a pripadne rozviceni cervenych svetel na maximalni intenzitu
		brakeLight(axg, ayg, azg, 0.25);
    //kotrola polohy jizdniho kola
		b = bikePosition(ax, ay, az, pos);
		//pokud bylo desetkrat vyhodnoceno, ze jizdni kolo neni v pohybu. tak se atmega a mpu uspi
		if (q>9)
		{
			q=0;
			sleep();
		}
		
		_delay_ms(50);
		
	}
}
//preruseni OVF pro spozdeni uspani cykloblikace
ISR (TIMER2_OVF_vect)
{
	cli();
	if (!b)
	{
		q++;
		b=0;
	}
	sei();
}
//presuseni od tlacitka pro znovu zavolani konfigurace
ISR (INT1_vect)
{
	cli();
	redLightOn(2500);
	_delay_ms(100);
	redLightOn(0);
	sei();
	sleep();
}

// preruseni OVF a COMP pro PWM
//OVF se stara o zapnuti svetel a behem vyvolani COMPA a COMPB dojde k jejich vypnuti
//COMPA ma na starost zluta svetla, COMPB ma na starost svetla cervena
ISR (TIMER1_OVF_vect)
{
	TCNT1 = 63036;
	PORTD |=(1<<PORTD5);
	PORTD |=(1<<PORTD7);
	PORTB |=(1<<PORTB3);
}

ISR (TIMER1_COMPA_vect)
{
	PORTD &=~(1<<PORTD5);
	PORTB &=~(1<<PORTB3);
}

ISR (TIMER1_COMPB_vect)
{
	PORTD ^=(1<<PORTD7);
}